"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstance = exports.prefetchConfig = exports.init = exports.EppoJSClient = void 0;
const js_client_sdk_common_1 = require("@eppo/js-client-sdk-common");
const sdk_data_1 = require("./sdk-data");
const vercel_async_store_1 = require("./vercel-async-store");
/**
 * Client for assigning experiment variations.
 * @public
 */
class EppoJSClient extends js_client_sdk_common_1.EppoClient {
    getStringAssignment(flagKey, subjectKey, subjectAttributes, defaultValue) {
        EppoJSClient.getAssignmentInitializationCheck();
        return super.getStringAssignment(flagKey, subjectKey, subjectAttributes, defaultValue);
    }
    getBoolAssignment(flagKey, subjectKey, subjectAttributes, defaultValue) {
        EppoJSClient.getAssignmentInitializationCheck();
        return super.getBoolAssignment(flagKey, subjectKey, subjectAttributes, defaultValue);
    }
    getIntegerAssignment(flagKey, subjectKey, subjectAttributes, defaultValue) {
        EppoJSClient.getAssignmentInitializationCheck();
        return super.getIntegerAssignment(flagKey, subjectKey, subjectAttributes, defaultValue);
    }
    getNumericAssignment(flagKey, subjectKey, subjectAttributes, defaultValue) {
        EppoJSClient.getAssignmentInitializationCheck();
        return super.getNumericAssignment(flagKey, subjectKey, subjectAttributes, defaultValue);
    }
    getJSONAssignment(flagKey, subjectKey, subjectAttributes, defaultValue) {
        EppoJSClient.getAssignmentInitializationCheck();
        return super.getJSONAssignment(flagKey, subjectKey, subjectAttributes, defaultValue);
    }
    static getAssignmentInitializationCheck() {
        if (!EppoJSClient.initialized) {
            console.warn('Eppo SDK assignment requested before init() completed');
        }
    }
}
exports.EppoJSClient = EppoJSClient;
// Ensure that the client is instantiated during class loading.
// Use an empty memory-only configuration store until the `init` method is called,
// to avoid serving stale data to the user.
EppoJSClient.instance = new EppoJSClient(new js_client_sdk_common_1.MemoryOnlyConfigurationStore(), undefined, true);
EppoJSClient.initialized = false;
/**
 * Initializes the Eppo client with configuration parameters.
 * This method should be called once on application startup.
 * @param config - client configuration
 * @public
 */
async function init(config) {
    await initClient(config);
    EppoJSClient.initialized = true;
    return EppoJSClient.instance;
}
exports.init = init;
async function initClient(config) {
    var _a, _b, _c, _d;
    validateConfig(config);
    try {
        // If any existing instances; ensure they are not polling
        if (EppoJSClient.instance) {
            EppoJSClient.instance.stopPolling();
        }
        const configurationStore = new js_client_sdk_common_1.HybridConfigurationStore(new js_client_sdk_common_1.MemoryStore(), new vercel_async_store_1.default(config.vercelParams.edgeConfig, config.vercelParams.edgeConfigStoreId, config.vercelParams.vercelApiToken, config.vercelParams.edgeConfigExpirationSeconds));
        await configurationStore.init();
        EppoJSClient.instance.setConfigurationStore(configurationStore);
        EppoJSClient.instance.useNonExpiringInMemoryAssignmentCache();
        const requestConfiguration = {
            apiKey: config.apiKey,
            sdkName: sdk_data_1.sdkName,
            sdkVersion: sdk_data_1.sdkVersion,
            baseUrl: (_a = config.baseUrl) !== null && _a !== void 0 ? _a : undefined,
            requestTimeoutMs: (_b = config.requestTimeoutMs) !== null && _b !== void 0 ? _b : undefined,
            numInitialRequestRetries: (_c = config.numInitialRequestRetries) !== null && _c !== void 0 ? _c : undefined,
            pollAfterSuccessfulInitialization: false,
            pollAfterFailedInitialization: false,
            throwOnFailedInitialization: true,
            skipInitialPoll: false,
        };
        EppoJSClient.instance.setLogger(config.assignmentLogger);
        EppoJSClient.instance.setConfigurationRequestParameters(requestConfiguration);
        if (config.vercelParams.vercelFunctionUrl) {
            const isConfigExpired = await configurationStore.isExpired();
            if (isConfigExpired) {
                fetch(config.vercelParams.vercelFunctionUrl);
            }
        }
    }
    catch (error) {
        console.warn('Eppo SDK encountered an error initializing, assignment calls will return the default value and not be logged');
        if ((_d = config.throwOnFailedInitialization) !== null && _d !== void 0 ? _d : true) {
            throw error;
        }
    }
}
async function prefetchConfig(config) {
    await initClient(config);
    await EppoJSClient.instance.fetchFlagConfigurations();
}
exports.prefetchConfig = prefetchConfig;
function validateConfig(config) {
    js_client_sdk_common_1.validation.validateNotBlank(config.apiKey, 'API key required');
    js_client_sdk_common_1.validation.validateNotBlank(config.vercelParams.edgeConfig, 'EDGE_CONFIG is required');
    js_client_sdk_common_1.validation.validateNotBlank(config.vercelParams.vercelApiToken, 'Vercel api token is required');
    js_client_sdk_common_1.validation.validateNotBlank(config.vercelParams.edgeConfigStoreId, 'Edge Config Store Id is required');
}
/**
 * Used to access a singleton SDK client instance.
 * Use the method after calling init() to initialize the client.
 * @returns a singleton client instance
 * @public
 */
function getInstance() {
    return EppoJSClient.instance;
}
exports.getInstance = getInstance;
//# sourceMappingURL=index.js.map